/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __CC_GLOBAL_SHORTCUTS_PROVIDER_GENERATED_H__
#define __CC_GLOBAL_SHORTCUTS_PROVIDER_GENERATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Settings.GlobalShortcutsProvider */

#define CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER (cc_settings_global_shortcuts_provider_get_type ())
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER, CcSettingsGlobalShortcutsProvider))
#define CC_IS_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER))
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER, CcSettingsGlobalShortcutsProviderIface))

struct _CcSettingsGlobalShortcutsProvider;
typedef struct _CcSettingsGlobalShortcutsProvider CcSettingsGlobalShortcutsProvider;
typedef struct _CcSettingsGlobalShortcutsProviderIface CcSettingsGlobalShortcutsProviderIface;

struct _CcSettingsGlobalShortcutsProviderIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_bind_shortcuts) (
    CcSettingsGlobalShortcutsProvider *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    GVariant *arg_shortcuts);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcSettingsGlobalShortcutsProvider, g_object_unref)
#endif

GType cc_settings_global_shortcuts_provider_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *cc_settings_global_shortcuts_provider_interface_info (void);
guint cc_settings_global_shortcuts_provider_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void cc_settings_global_shortcuts_provider_complete_bind_shortcuts (
    CcSettingsGlobalShortcutsProvider *object,
    GDBusMethodInvocation *invocation,
    GVariant *results);



/* D-Bus method calls: */
void cc_settings_global_shortcuts_provider_call_bind_shortcuts (
    CcSettingsGlobalShortcutsProvider *proxy,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    GVariant *arg_shortcuts,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean cc_settings_global_shortcuts_provider_call_bind_shortcuts_finish (
    CcSettingsGlobalShortcutsProvider *proxy,
    GVariant ** out_results,
    GAsyncResult *res,
    GError **error);

gboolean cc_settings_global_shortcuts_provider_call_bind_shortcuts_sync (
    CcSettingsGlobalShortcutsProvider *proxy,
    const gchar *arg_app_id,
    const gchar *arg_parent_window,
    GVariant *arg_shortcuts,
    GVariant ** out_results,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY (cc_settings_global_shortcuts_provider_proxy_get_type ())
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY, CcSettingsGlobalShortcutsProviderProxy))
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY, CcSettingsGlobalShortcutsProviderProxyClass))
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY, CcSettingsGlobalShortcutsProviderProxyClass))
#define CC_IS_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY))
#define CC_IS_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_PROXY))

typedef struct _CcSettingsGlobalShortcutsProviderProxy CcSettingsGlobalShortcutsProviderProxy;
typedef struct _CcSettingsGlobalShortcutsProviderProxyClass CcSettingsGlobalShortcutsProviderProxyClass;
typedef struct _CcSettingsGlobalShortcutsProviderProxyPrivate CcSettingsGlobalShortcutsProviderProxyPrivate;

struct _CcSettingsGlobalShortcutsProviderProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  CcSettingsGlobalShortcutsProviderProxyPrivate *priv;
};

struct _CcSettingsGlobalShortcutsProviderProxyClass
{
  GDBusProxyClass parent_class;
};

GType cc_settings_global_shortcuts_provider_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcSettingsGlobalShortcutsProviderProxy, g_object_unref)
#endif

void cc_settings_global_shortcuts_provider_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcSettingsGlobalShortcutsProvider *cc_settings_global_shortcuts_provider_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
CcSettingsGlobalShortcutsProvider *cc_settings_global_shortcuts_provider_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void cc_settings_global_shortcuts_provider_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcSettingsGlobalShortcutsProvider *cc_settings_global_shortcuts_provider_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
CcSettingsGlobalShortcutsProvider *cc_settings_global_shortcuts_provider_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON (cc_settings_global_shortcuts_provider_skeleton_get_type ())
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON, CcSettingsGlobalShortcutsProviderSkeleton))
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON, CcSettingsGlobalShortcutsProviderSkeletonClass))
#define CC_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON, CcSettingsGlobalShortcutsProviderSkeletonClass))
#define CC_IS_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON))
#define CC_IS_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_TYPE_SETTINGS_GLOBAL_SHORTCUTS_PROVIDER_SKELETON))

typedef struct _CcSettingsGlobalShortcutsProviderSkeleton CcSettingsGlobalShortcutsProviderSkeleton;
typedef struct _CcSettingsGlobalShortcutsProviderSkeletonClass CcSettingsGlobalShortcutsProviderSkeletonClass;
typedef struct _CcSettingsGlobalShortcutsProviderSkeletonPrivate CcSettingsGlobalShortcutsProviderSkeletonPrivate;

struct _CcSettingsGlobalShortcutsProviderSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  CcSettingsGlobalShortcutsProviderSkeletonPrivate *priv;
};

struct _CcSettingsGlobalShortcutsProviderSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType cc_settings_global_shortcuts_provider_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcSettingsGlobalShortcutsProviderSkeleton, g_object_unref)
#endif

CcSettingsGlobalShortcutsProvider *cc_settings_global_shortcuts_provider_skeleton_new (void);


G_END_DECLS

#endif /* __CC_GLOBAL_SHORTCUTS_PROVIDER_GENERATED_H__ */
