
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "cc-background-enum-types.h"
#include "cc-background-item.h"
#include "cc-background-paintable.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "cc-background-item.h" */

GType
cc_background_item_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_SHADING), "CC_BACKGROUND_ITEM_HAS_SHADING", "shading" },
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_PLACEMENT), "CC_BACKGROUND_ITEM_HAS_PLACEMENT", "placement" },
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_PCOLOR), "CC_BACKGROUND_ITEM_HAS_PCOLOR", "pcolor" },
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_SCOLOR), "CC_BACKGROUND_ITEM_HAS_SCOLOR", "scolor" },
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_URI), "CC_BACKGROUND_ITEM_HAS_URI", "uri" },
        { C_FLAGS (CC_BACKGROUND_ITEM_HAS_URI_DARK), "CC_BACKGROUND_ITEM_HAS_URI_DARK", "uri-dark" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("CcBackgroundItemFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "cc-background-paintable.h" */

GType
cc_background_paint_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (CC_BACKGROUND_PAINT_LIGHT), "CC_BACKGROUND_PAINT_LIGHT", "light" },
        { C_FLAGS (CC_BACKGROUND_PAINT_DARK), "CC_BACKGROUND_PAINT_DARK", "dark" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("CcBackgroundPaintFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

