
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "gvc-mixer-control.h" */

GType gvc_mixer_control_state_get_type (void);
#define GVC_TYPE_MIXER_CONTROL_STATE (gvc_mixer_control_state_get_type())

GType gvc_headset_port_choice_get_type (void);
#define GVC_TYPE_HEADSET_PORT_CHOICE (gvc_headset_port_choice_get_type())

/* enumerations from "gvc-mixer-stream.h" */

GType gvc_mixer_stream_state_get_type (void);
#define GVC_TYPE_MIXER_STREAM_STATE (gvc_mixer_stream_state_get_type())

/* enumerations from "gvc-mixer-ui-device.h" */

GType gvc_mixer_ui_device_direction_get_type (void);
#define GVC_TYPE_MIXER_UI_DEVICE_DIRECTION (gvc_mixer_ui_device_direction_get_type())

G_END_DECLS

/* Generated data ends here */

