
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "bolt-enum-types.h"
#include "bolt/bolt-enums.h"
#include "bolt/bolt-error.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "bolt-enums.h" */

GType
bolt_status_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_STATUS_UNKNOWN), "BOLT_STATUS_UNKNOWN", "unknown" },
        { C_ENUM (BOLT_STATUS_DISCONNECTED), "BOLT_STATUS_DISCONNECTED", "disconnected" },
        { C_ENUM (BOLT_STATUS_CONNECTING), "BOLT_STATUS_CONNECTING", "connecting" },
        { C_ENUM (BOLT_STATUS_CONNECTED), "BOLT_STATUS_CONNECTED", "connected" },
        { C_ENUM (BOLT_STATUS_AUTHORIZING), "BOLT_STATUS_AUTHORIZING", "authorizing" },
        { C_ENUM (BOLT_STATUS_AUTH_ERROR), "BOLT_STATUS_AUTH_ERROR", "auth-error" },
        { C_ENUM (BOLT_STATUS_AUTHORIZED), "BOLT_STATUS_AUTHORIZED", "authorized" },
        { C_ENUM (BOLT_STATUS_AUTHORIZED_SECURE), "BOLT_STATUS_AUTHORIZED_SECURE", "authorized-secure" },
        { C_ENUM (BOLT_STATUS_AUTHORIZED_NEWKEY), "BOLT_STATUS_AUTHORIZED_NEWKEY", "authorized-newkey" },
        { C_ENUM (BOLT_STATUS_AUTHORIZED_DPONLY), "BOLT_STATUS_AUTHORIZED_DPONLY", "authorized-dponly" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltStatus"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_auth_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (BOLT_AUTH_NONE), "BOLT_AUTH_NONE", "none" },
        { C_FLAGS (BOLT_AUTH_NOPCIE), "BOLT_AUTH_NOPCIE", "nopcie" },
        { C_FLAGS (BOLT_AUTH_SECURE), "BOLT_AUTH_SECURE", "secure" },
        { C_FLAGS (BOLT_AUTH_NOKEY), "BOLT_AUTH_NOKEY", "nokey" },
        { C_FLAGS (BOLT_AUTH_BOOT), "BOLT_AUTH_BOOT", "boot" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("BoltAuthFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_key_state_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_KEY_UNKNOWN), "BOLT_KEY_UNKNOWN", "unknown" },
        { C_ENUM (BOLT_KEY_MISSING), "BOLT_KEY_MISSING", "missing" },
        { C_ENUM (BOLT_KEY_HAVE), "BOLT_KEY_HAVE", "have" },
        { C_ENUM (BOLT_KEY_NEW), "BOLT_KEY_NEW", "new" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltKeyState"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_security_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_SECURITY_UNKNOWN), "BOLT_SECURITY_UNKNOWN", "unknown" },
        { C_ENUM (BOLT_SECURITY_NONE), "BOLT_SECURITY_NONE", "none" },
        { C_ENUM (BOLT_SECURITY_DPONLY), "BOLT_SECURITY_DPONLY", "dponly" },
        { C_ENUM (BOLT_SECURITY_USER), "BOLT_SECURITY_USER", "user" },
        { C_ENUM (BOLT_SECURITY_SECURE), "BOLT_SECURITY_SECURE", "secure" },
        { C_ENUM (BOLT_SECURITY_USBONLY), "BOLT_SECURITY_USBONLY", "usbonly" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltSecurity"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_policy_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_POLICY_UNKNOWN), "BOLT_POLICY_UNKNOWN", "unknown" },
        { C_ENUM (BOLT_POLICY_DEFAULT), "BOLT_POLICY_DEFAULT", "default" },
        { C_ENUM (BOLT_POLICY_MANUAL), "BOLT_POLICY_MANUAL", "manual" },
        { C_ENUM (BOLT_POLICY_AUTO), "BOLT_POLICY_AUTO", "auto" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltPolicy"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_auth_ctrl_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (BOLT_AUTHCTRL_NONE), "BOLT_AUTHCTRL_NONE", "none" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("BoltAuthCtrl"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_device_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_DEVICE_UNKNOWN_TYPE), "BOLT_DEVICE_UNKNOWN_TYPE", "unknown-type" },
        { C_ENUM (BOLT_DEVICE_HOST), "BOLT_DEVICE_HOST", "host" },
        { C_ENUM (BOLT_DEVICE_PERIPHERAL), "BOLT_DEVICE_PERIPHERAL", "peripheral" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltDeviceType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
bolt_auth_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (BOLT_AUTH_DISABLED), "BOLT_AUTH_DISABLED", "disabled" },
        { C_FLAGS (BOLT_AUTH_ENABLED), "BOLT_AUTH_ENABLED", "enabled" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("BoltAuthMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "bolt-error.h" */

GType
bolt_error_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (BOLT_ERROR_FAILED), "BOLT_ERROR_FAILED", "failed" },
        { C_ENUM (BOLT_ERROR_UDEV), "BOLT_ERROR_UDEV", "udev" },
        { C_ENUM (BOLT_ERROR_NOKEY), "BOLT_ERROR_NOKEY", "nokey" },
        { C_ENUM (BOLT_ERROR_BADKEY), "BOLT_ERROR_BADKEY", "badkey" },
        { C_ENUM (BOLT_ERROR_CFG), "BOLT_ERROR_CFG", "cfg" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("BoltError"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

