
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gdesktop-enums-types.h"
#include "../../../../../usr/include/gsettings-desktop-schemas/gdesktop-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "gdesktop-enums.h" */

GType
g_desktop_proxy_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_PROXY_MODE_NONE), "G_DESKTOP_PROXY_MODE_NONE", "none" },
        { C_ENUM (G_DESKTOP_PROXY_MODE_MANUAL), "G_DESKTOP_PROXY_MODE_MANUAL", "manual" },
        { C_ENUM (G_DESKTOP_PROXY_MODE_AUTO), "G_DESKTOP_PROXY_MODE_AUTO", "auto" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopProxyMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_toolbar_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TOOLBAR_STYLE_BOTH), "G_DESKTOP_TOOLBAR_STYLE_BOTH", "both" },
        { C_ENUM (G_DESKTOP_TOOLBAR_STYLE_BOTH_HORIZ), "G_DESKTOP_TOOLBAR_STYLE_BOTH_HORIZ", "both-horiz" },
        { C_ENUM (G_DESKTOP_TOOLBAR_STYLE_ICONS), "G_DESKTOP_TOOLBAR_STYLE_ICONS", "icons" },
        { C_ENUM (G_DESKTOP_TOOLBAR_STYLE_TEXT), "G_DESKTOP_TOOLBAR_STYLE_TEXT", "text" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopToolbarStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_toolbar_icon_size_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TOOLBAR_ICON_SIZE_SMALL), "G_DESKTOP_TOOLBAR_ICON_SIZE_SMALL", "small" },
        { C_ENUM (G_DESKTOP_TOOLBAR_ICON_SIZE_LARGE), "G_DESKTOP_TOOLBAR_ICON_SIZE_LARGE", "large" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopToolbarIconSize"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_background_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_NONE), "G_DESKTOP_BACKGROUND_STYLE_NONE", "none" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_WALLPAPER), "G_DESKTOP_BACKGROUND_STYLE_WALLPAPER", "wallpaper" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_CENTERED), "G_DESKTOP_BACKGROUND_STYLE_CENTERED", "centered" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_SCALED), "G_DESKTOP_BACKGROUND_STYLE_SCALED", "scaled" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_STRETCHED), "G_DESKTOP_BACKGROUND_STYLE_STRETCHED", "stretched" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_ZOOM), "G_DESKTOP_BACKGROUND_STYLE_ZOOM", "zoom" },
        { C_ENUM (G_DESKTOP_BACKGROUND_STYLE_SPANNED), "G_DESKTOP_BACKGROUND_STYLE_SPANNED", "spanned" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopBackgroundStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_background_shading_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_BACKGROUND_SHADING_SOLID), "G_DESKTOP_BACKGROUND_SHADING_SOLID", "solid" },
        { C_ENUM (G_DESKTOP_BACKGROUND_SHADING_VERTICAL), "G_DESKTOP_BACKGROUND_SHADING_VERTICAL", "vertical" },
        { C_ENUM (G_DESKTOP_BACKGROUND_SHADING_HORIZONTAL), "G_DESKTOP_BACKGROUND_SHADING_HORIZONTAL", "horizontal" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopBackgroundShading"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_mouse_dwell_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_MODE_WINDOW), "G_DESKTOP_MOUSE_DWELL_MODE_WINDOW", "window" },
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_MODE_GESTURE), "G_DESKTOP_MOUSE_DWELL_MODE_GESTURE", "gesture" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMouseDwellMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_mouse_dwell_direction_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_DIRECTION_LEFT), "G_DESKTOP_MOUSE_DWELL_DIRECTION_LEFT", "left" },
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_DIRECTION_RIGHT), "G_DESKTOP_MOUSE_DWELL_DIRECTION_RIGHT", "right" },
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_DIRECTION_UP), "G_DESKTOP_MOUSE_DWELL_DIRECTION_UP", "up" },
        { C_ENUM (G_DESKTOP_MOUSE_DWELL_DIRECTION_DOWN), "G_DESKTOP_MOUSE_DWELL_DIRECTION_DOWN", "down" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMouseDwellDirection"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_clock_format_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_CLOCK_FORMAT_24H), "G_DESKTOP_CLOCK_FORMAT_24H", "24h" },
        { C_ENUM (G_DESKTOP_CLOCK_FORMAT_12H), "G_DESKTOP_CLOCK_FORMAT_12H", "12h" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopClockFormat"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_screensaver_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_SCREENSAVER_MODE_BLANK_ONLY), "G_DESKTOP_SCREENSAVER_MODE_BLANK_ONLY", "blank-only" },
        { C_ENUM (G_DESKTOP_SCREENSAVER_MODE_RANDOM), "G_DESKTOP_SCREENSAVER_MODE_RANDOM", "random" },
        { C_ENUM (G_DESKTOP_SCREENSAVER_MODE_SINGLE), "G_DESKTOP_SCREENSAVER_MODE_SINGLE", "single" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopScreensaverMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_magnifier_mouse_tracking_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_NONE), "G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_NONE", "none" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_CENTERED), "G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_CENTERED", "centered" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_PROPORTIONAL), "G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_PROPORTIONAL", "proportional" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_PUSH), "G_DESKTOP_MAGNIFIER_MOUSE_TRACKING_MODE_PUSH", "push" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMagnifierMouseTrackingMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_magnifier_focus_tracking_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_NONE), "G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_NONE", "none" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_CENTERED), "G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_CENTERED", "centered" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_PROPORTIONAL), "G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_PROPORTIONAL", "proportional" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_PUSH), "G_DESKTOP_MAGNIFIER_FOCUS_TRACKING_MODE_PUSH", "push" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMagnifierFocusTrackingMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_magnifier_caret_tracking_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_NONE), "G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_NONE", "none" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_CENTERED), "G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_CENTERED", "centered" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_PROPORTIONAL), "G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_PROPORTIONAL", "proportional" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_PUSH), "G_DESKTOP_MAGNIFIER_CARET_TRACKING_MODE_PUSH", "push" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMagnifierCaretTrackingMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_magnifier_screen_position_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_NONE), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_NONE", "none" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_FULL_SCREEN), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_FULL_SCREEN", "full-screen" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_TOP_HALF), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_TOP_HALF", "top-half" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_BOTTOM_HALF), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_BOTTOM_HALF", "bottom-half" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_LEFT_HALF), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_LEFT_HALF", "left-half" },
        { C_ENUM (G_DESKTOP_MAGNIFIER_SCREEN_POSITION_RIGHT_HALF), "G_DESKTOP_MAGNIFIER_SCREEN_POSITION_RIGHT_HALF", "right-half" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopMagnifierScreenPosition"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_titlebar_action_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_TOGGLE_SHADE), "G_DESKTOP_TITLEBAR_ACTION_TOGGLE_SHADE", "toggle-shade" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE), "G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE", "toggle-maximize" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE_HORIZONTALLY), "G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE_HORIZONTALLY", "toggle-maximize-horizontally" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE_VERTICALLY), "G_DESKTOP_TITLEBAR_ACTION_TOGGLE_MAXIMIZE_VERTICALLY", "toggle-maximize-vertically" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_MINIMIZE), "G_DESKTOP_TITLEBAR_ACTION_MINIMIZE", "minimize" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_NONE), "G_DESKTOP_TITLEBAR_ACTION_NONE", "none" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_LOWER), "G_DESKTOP_TITLEBAR_ACTION_LOWER", "lower" },
        { C_ENUM (G_DESKTOP_TITLEBAR_ACTION_MENU), "G_DESKTOP_TITLEBAR_ACTION_MENU", "menu" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopTitlebarAction"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_focus_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FOCUS_MODE_CLICK), "G_DESKTOP_FOCUS_MODE_CLICK", "click" },
        { C_ENUM (G_DESKTOP_FOCUS_MODE_SLOPPY), "G_DESKTOP_FOCUS_MODE_SLOPPY", "sloppy" },
        { C_ENUM (G_DESKTOP_FOCUS_MODE_MOUSE), "G_DESKTOP_FOCUS_MODE_MOUSE", "mouse" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFocusMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_focus_new_windows_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FOCUS_NEW_WINDOWS_SMART), "G_DESKTOP_FOCUS_NEW_WINDOWS_SMART", "smart" },
        { C_ENUM (G_DESKTOP_FOCUS_NEW_WINDOWS_STRICT), "G_DESKTOP_FOCUS_NEW_WINDOWS_STRICT", "strict" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFocusNewWindows"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_visual_bell_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_VISUAL_BELL_FULLSCREEN_FLASH), "G_DESKTOP_VISUAL_BELL_FULLSCREEN_FLASH", "fullscreen-flash" },
        { C_ENUM (G_DESKTOP_VISUAL_BELL_FRAME_FLASH), "G_DESKTOP_VISUAL_BELL_FRAME_FLASH", "frame-flash" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopVisualBellType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_location_accuracy_level_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_LOCATION_ACCURACY_LEVEL_COUNTRY), "G_DESKTOP_LOCATION_ACCURACY_LEVEL_COUNTRY", "country" },
        { C_ENUM (G_DESKTOP_LOCATION_ACCURACY_LEVEL_CITY), "G_DESKTOP_LOCATION_ACCURACY_LEVEL_CITY", "city" },
        { C_ENUM (G_DESKTOP_LOCATION_ACCURACY_LEVEL_NEIGHBORHOOD), "G_DESKTOP_LOCATION_ACCURACY_LEVEL_NEIGHBORHOOD", "neighborhood" },
        { C_ENUM (G_DESKTOP_LOCATION_ACCURACY_LEVEL_STREET), "G_DESKTOP_LOCATION_ACCURACY_LEVEL_STREET", "street" },
        { C_ENUM (G_DESKTOP_LOCATION_ACCURACY_LEVEL_EXACT), "G_DESKTOP_LOCATION_ACCURACY_LEVEL_EXACT", "exact" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopLocationAccuracyLevel"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_touchpad_handedness_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TOUCHPAD_HANDEDNESS_RIGHT), "G_DESKTOP_TOUCHPAD_HANDEDNESS_RIGHT", "right" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_HANDEDNESS_LEFT), "G_DESKTOP_TOUCHPAD_HANDEDNESS_LEFT", "left" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_HANDEDNESS_MOUSE), "G_DESKTOP_TOUCHPAD_HANDEDNESS_MOUSE", "mouse" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopTouchpadHandedness"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_touchpad_click_method_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TOUCHPAD_CLICK_METHOD_DEFAULT), "G_DESKTOP_TOUCHPAD_CLICK_METHOD_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_CLICK_METHOD_NONE), "G_DESKTOP_TOUCHPAD_CLICK_METHOD_NONE", "none" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_CLICK_METHOD_AREAS), "G_DESKTOP_TOUCHPAD_CLICK_METHOD_AREAS", "areas" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_CLICK_METHOD_FINGERS), "G_DESKTOP_TOUCHPAD_CLICK_METHOD_FINGERS", "fingers" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopTouchpadClickMethod"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_touchpad_tap_button_map_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_DEFAULT), "G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_LRM), "G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_LRM", "lrm" },
        { C_ENUM (G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_LMR), "G_DESKTOP_TOUCHPAD_BUTTON_TAP_MAP_LMR", "lmr" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopTouchpadTapButtonMap"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_device_send_events_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_DEVICE_SEND_EVENTS_ENABLED), "G_DESKTOP_DEVICE_SEND_EVENTS_ENABLED", "enabled" },
        { C_ENUM (G_DESKTOP_DEVICE_SEND_EVENTS_DISABLED), "G_DESKTOP_DEVICE_SEND_EVENTS_DISABLED", "disabled" },
        { C_ENUM (G_DESKTOP_DEVICE_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE), "G_DESKTOP_DEVICE_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE", "disabled-on-external-mouse" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopDeviceSendEvents"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_tablet_mapping_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_TABLET_MAPPING_ABSOLUTE), "G_DESKTOP_TABLET_MAPPING_ABSOLUTE", "absolute" },
        { C_ENUM (G_DESKTOP_TABLET_MAPPING_RELATIVE), "G_DESKTOP_TABLET_MAPPING_RELATIVE", "relative" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopTabletMapping"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_pad_button_action_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_PAD_BUTTON_ACTION_NONE), "G_DESKTOP_PAD_BUTTON_ACTION_NONE", "none" },
        { C_ENUM (G_DESKTOP_PAD_BUTTON_ACTION_HELP), "G_DESKTOP_PAD_BUTTON_ACTION_HELP", "help" },
        { C_ENUM (G_DESKTOP_PAD_BUTTON_ACTION_SWITCH_MONITOR), "G_DESKTOP_PAD_BUTTON_ACTION_SWITCH_MONITOR", "switch-monitor" },
        { C_ENUM (G_DESKTOP_PAD_BUTTON_ACTION_KEYBINDING), "G_DESKTOP_PAD_BUTTON_ACTION_KEYBINDING", "keybinding" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopPadButtonAction"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_stylus_button_action_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_DEFAULT), "G_DESKTOP_STYLUS_BUTTON_ACTION_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_MIDDLE), "G_DESKTOP_STYLUS_BUTTON_ACTION_MIDDLE", "middle" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_RIGHT), "G_DESKTOP_STYLUS_BUTTON_ACTION_RIGHT", "right" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_BACK), "G_DESKTOP_STYLUS_BUTTON_ACTION_BACK", "back" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_FORWARD), "G_DESKTOP_STYLUS_BUTTON_ACTION_FORWARD", "forward" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_SWITCH_MONITOR), "G_DESKTOP_STYLUS_BUTTON_ACTION_SWITCH_MONITOR", "switch-monitor" },
        { C_ENUM (G_DESKTOP_STYLUS_BUTTON_ACTION_KEYBINDING), "G_DESKTOP_STYLUS_BUTTON_ACTION_KEYBINDING", "keybinding" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopStylusButtonAction"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_stylus_eraser_button_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_STYLUS_ERASER_BUTTON_MODE_DEFAULT), "G_DESKTOP_STYLUS_ERASER_BUTTON_MODE_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_STYLUS_ERASER_BUTTON_MODE_BUTTON), "G_DESKTOP_STYLUS_ERASER_BUTTON_MODE_BUTTON", "button" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopStylusEraserButtonMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_pointer_accel_profile_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_POINTER_ACCEL_PROFILE_DEFAULT), "G_DESKTOP_POINTER_ACCEL_PROFILE_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_POINTER_ACCEL_PROFILE_FLAT), "G_DESKTOP_POINTER_ACCEL_PROFILE_FLAT", "flat" },
        { C_ENUM (G_DESKTOP_POINTER_ACCEL_PROFILE_ADAPTIVE), "G_DESKTOP_POINTER_ACCEL_PROFILE_ADAPTIVE", "adaptive" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopPointerAccelProfile"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_usb_protection_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_USB_PROTECTION_LOCKSCREEN), "G_DESKTOP_USB_PROTECTION_LOCKSCREEN", "lockscreen" },
        { C_ENUM (G_DESKTOP_USB_PROTECTION_ALWAYS), "G_DESKTOP_USB_PROTECTION_ALWAYS", "always" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopUsbProtection"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_font_antialiasing_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FONT_ANTIALIASING_MODE_NONE), "G_DESKTOP_FONT_ANTIALIASING_MODE_NONE", "none" },
        { C_ENUM (G_DESKTOP_FONT_ANTIALIASING_MODE_GRAYSCALE), "G_DESKTOP_FONT_ANTIALIASING_MODE_GRAYSCALE", "grayscale" },
        { C_ENUM (G_DESKTOP_FONT_ANTIALIASING_MODE_RGBA), "G_DESKTOP_FONT_ANTIALIASING_MODE_RGBA", "rgba" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFontAntialiasingMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_font_hinting_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FONT_HINTING_NONE), "G_DESKTOP_FONT_HINTING_NONE", "none" },
        { C_ENUM (G_DESKTOP_FONT_HINTING_SLIGHT), "G_DESKTOP_FONT_HINTING_SLIGHT", "slight" },
        { C_ENUM (G_DESKTOP_FONT_HINTING_MEDIUM), "G_DESKTOP_FONT_HINTING_MEDIUM", "medium" },
        { C_ENUM (G_DESKTOP_FONT_HINTING_FULL), "G_DESKTOP_FONT_HINTING_FULL", "full" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFontHinting"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_font_rgba_order_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FONT_RGBA_ORDER_RGBA), "G_DESKTOP_FONT_RGBA_ORDER_RGBA", "rgba" },
        { C_ENUM (G_DESKTOP_FONT_RGBA_ORDER_RGB), "G_DESKTOP_FONT_RGBA_ORDER_RGB", "rgb" },
        { C_ENUM (G_DESKTOP_FONT_RGBA_ORDER_BGR), "G_DESKTOP_FONT_RGBA_ORDER_BGR", "bgr" },
        { C_ENUM (G_DESKTOP_FONT_RGBA_ORDER_VRGB), "G_DESKTOP_FONT_RGBA_ORDER_VRGB", "vrgb" },
        { C_ENUM (G_DESKTOP_FONT_RGBA_ORDER_VBGR), "G_DESKTOP_FONT_RGBA_ORDER_VBGR", "vbgr" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFontRgbaOrder"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_color_scheme_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_COLOR_SCHEME_DEFAULT), "G_DESKTOP_COLOR_SCHEME_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_COLOR_SCHEME_PREFER_DARK), "G_DESKTOP_COLOR_SCHEME_PREFER_DARK", "prefer-dark" },
        { C_ENUM (G_DESKTOP_COLOR_SCHEME_PREFER_LIGHT), "G_DESKTOP_COLOR_SCHEME_PREFER_LIGHT", "prefer-light" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopColorScheme"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_pointing_stick_scroll_method_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_POINTING_STICK_SCROLL_METHOD_DEFAULT), "G_DESKTOP_POINTING_STICK_SCROLL_METHOD_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_POINTING_STICK_SCROLL_METHOD_NONE), "G_DESKTOP_POINTING_STICK_SCROLL_METHOD_NONE", "none" },
        { C_ENUM (G_DESKTOP_POINTING_STICK_SCROLL_METHOD_ON_BUTTON_DOWN), "G_DESKTOP_POINTING_STICK_SCROLL_METHOD_ON_BUTTON_DOWN", "on-button-down" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopPointingStickScrollMethod"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_font_rendering_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_FONT_RENDERING_AUTOMATIC), "G_DESKTOP_FONT_RENDERING_AUTOMATIC", "automatic" },
        { C_ENUM (G_DESKTOP_FONT_RENDERING_MANUAL), "G_DESKTOP_FONT_RENDERING_MANUAL", "manual" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopFontRendering"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_accent_color_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_BLUE), "G_DESKTOP_ACCENT_COLOR_BLUE", "blue" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_TEAL), "G_DESKTOP_ACCENT_COLOR_TEAL", "teal" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_GREEN), "G_DESKTOP_ACCENT_COLOR_GREEN", "green" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_YELLOW), "G_DESKTOP_ACCENT_COLOR_YELLOW", "yellow" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_ORANGE), "G_DESKTOP_ACCENT_COLOR_ORANGE", "orange" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_RED), "G_DESKTOP_ACCENT_COLOR_RED", "red" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_PINK), "G_DESKTOP_ACCENT_COLOR_PINK", "pink" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_PURPLE), "G_DESKTOP_ACCENT_COLOR_PURPLE", "purple" },
        { C_ENUM (G_DESKTOP_ACCENT_COLOR_SLATE), "G_DESKTOP_ACCENT_COLOR_SLATE", "slate" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopAccentColor"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_reduced_motion_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_REDUCED_MOTION_NO_PREFERENCE), "G_DESKTOP_REDUCED_MOTION_NO_PREFERENCE", "no-preference" },
        { C_ENUM (G_DESKTOP_REDUCED_MOTION_REDUCE), "G_DESKTOP_REDUCED_MOTION_REDUCE", "reduce" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopReducedMotion"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
g_desktop_weekday_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (G_DESKTOP_WEEKDAY_DEFAULT), "G_DESKTOP_WEEKDAY_DEFAULT", "default" },
        { C_ENUM (G_DESKTOP_WEEKDAY_MONDAY), "G_DESKTOP_WEEKDAY_MONDAY", "monday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_TUESDAY), "G_DESKTOP_WEEKDAY_TUESDAY", "tuesday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_WEDNESDAY), "G_DESKTOP_WEEKDAY_WEDNESDAY", "wednesday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_THURSDAY), "G_DESKTOP_WEEKDAY_THURSDAY", "thursday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_FRIDAY), "G_DESKTOP_WEEKDAY_FRIDAY", "friday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_SATURDAY), "G_DESKTOP_WEEKDAY_SATURDAY", "saturday" },
        { C_ENUM (G_DESKTOP_WEEKDAY_SUNDAY), "G_DESKTOP_WEEKDAY_SUNDAY", "sunday" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("GDesktopWeekday"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

