
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "gsd-common-enums.h"
#include "gsd-device-manager.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "gsd-device-manager.h" */

GType
gsd_device_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (GSD_DEVICE_TYPE_MOUSE), "GSD_DEVICE_TYPE_MOUSE", "mouse" },
        { C_FLAGS (GSD_DEVICE_TYPE_KEYBOARD), "GSD_DEVICE_TYPE_KEYBOARD", "keyboard" },
        { C_FLAGS (GSD_DEVICE_TYPE_TOUCHPAD), "GSD_DEVICE_TYPE_TOUCHPAD", "touchpad" },
        { C_FLAGS (GSD_DEVICE_TYPE_TABLET), "GSD_DEVICE_TYPE_TABLET", "tablet" },
        { C_FLAGS (GSD_DEVICE_TYPE_TOUCHSCREEN), "GSD_DEVICE_TYPE_TOUCHSCREEN", "touchscreen" },
        { C_FLAGS (GSD_DEVICE_TYPE_PAD), "GSD_DEVICE_TYPE_PAD", "pad" },
        { C_FLAGS (GSD_DEVICE_TYPE_POINTINGSTICK), "GSD_DEVICE_TYPE_POINTINGSTICK", "pointingstick" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("GsdDeviceType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

