
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "cc-background-item.h" */

GType cc_background_item_flags_get_type (void);
#define CC_TYPE_BACKGROUND_ITEM_FLAGS (cc_background_item_flags_get_type())

/* enumerations from "cc-background-paintable.h" */

GType cc_background_paint_flags_get_type (void);
#define CC_TYPE_BACKGROUND_PAINT_FLAGS (cc_background_paint_flags_get_type())

G_END_DECLS

/* Generated data ends here */

