/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __CC_GLOBAL_SHORTCUTS_REBIND_GENERATED_H__
#define __CC_GLOBAL_SHORTCUTS_REBIND_GENERATED_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.GlobalShortcutsRebind */

#define CC_TYPE_GLOBAL_SHORTCUTS_REBIND (cc_global_shortcuts_rebind_get_type ())
#define CC_GLOBAL_SHORTCUTS_REBIND(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND, CcGlobalShortcutsRebind))
#define CC_IS_GLOBAL_SHORTCUTS_REBIND(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND))
#define CC_GLOBAL_SHORTCUTS_REBIND_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND, CcGlobalShortcutsRebindIface))

struct _CcGlobalShortcutsRebind;
typedef struct _CcGlobalShortcutsRebind CcGlobalShortcutsRebind;
typedef struct _CcGlobalShortcutsRebindIface CcGlobalShortcutsRebindIface;

struct _CcGlobalShortcutsRebindIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_rebind_shortcuts) (
    CcGlobalShortcutsRebind *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_app_id,
    GVariant *arg_shortcuts);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcGlobalShortcutsRebind, g_object_unref)
#endif

GType cc_global_shortcuts_rebind_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *cc_global_shortcuts_rebind_interface_info (void);
guint cc_global_shortcuts_rebind_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void cc_global_shortcuts_rebind_complete_rebind_shortcuts (
    CcGlobalShortcutsRebind *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void cc_global_shortcuts_rebind_call_rebind_shortcuts (
    CcGlobalShortcutsRebind *proxy,
    const gchar *arg_app_id,
    GVariant *arg_shortcuts,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean cc_global_shortcuts_rebind_call_rebind_shortcuts_finish (
    CcGlobalShortcutsRebind *proxy,
    GAsyncResult *res,
    GError **error);

gboolean cc_global_shortcuts_rebind_call_rebind_shortcuts_sync (
    CcGlobalShortcutsRebind *proxy,
    const gchar *arg_app_id,
    GVariant *arg_shortcuts,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY (cc_global_shortcuts_rebind_proxy_get_type ())
#define CC_GLOBAL_SHORTCUTS_REBIND_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY, CcGlobalShortcutsRebindProxy))
#define CC_GLOBAL_SHORTCUTS_REBIND_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY, CcGlobalShortcutsRebindProxyClass))
#define CC_GLOBAL_SHORTCUTS_REBIND_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY, CcGlobalShortcutsRebindProxyClass))
#define CC_IS_GLOBAL_SHORTCUTS_REBIND_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY))
#define CC_IS_GLOBAL_SHORTCUTS_REBIND_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_PROXY))

typedef struct _CcGlobalShortcutsRebindProxy CcGlobalShortcutsRebindProxy;
typedef struct _CcGlobalShortcutsRebindProxyClass CcGlobalShortcutsRebindProxyClass;
typedef struct _CcGlobalShortcutsRebindProxyPrivate CcGlobalShortcutsRebindProxyPrivate;

struct _CcGlobalShortcutsRebindProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  CcGlobalShortcutsRebindProxyPrivate *priv;
};

struct _CcGlobalShortcutsRebindProxyClass
{
  GDBusProxyClass parent_class;
};

GType cc_global_shortcuts_rebind_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcGlobalShortcutsRebindProxy, g_object_unref)
#endif

void cc_global_shortcuts_rebind_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcGlobalShortcutsRebind *cc_global_shortcuts_rebind_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
CcGlobalShortcutsRebind *cc_global_shortcuts_rebind_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void cc_global_shortcuts_rebind_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcGlobalShortcutsRebind *cc_global_shortcuts_rebind_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
CcGlobalShortcutsRebind *cc_global_shortcuts_rebind_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON (cc_global_shortcuts_rebind_skeleton_get_type ())
#define CC_GLOBAL_SHORTCUTS_REBIND_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON, CcGlobalShortcutsRebindSkeleton))
#define CC_GLOBAL_SHORTCUTS_REBIND_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON, CcGlobalShortcutsRebindSkeletonClass))
#define CC_GLOBAL_SHORTCUTS_REBIND_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON, CcGlobalShortcutsRebindSkeletonClass))
#define CC_IS_GLOBAL_SHORTCUTS_REBIND_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON))
#define CC_IS_GLOBAL_SHORTCUTS_REBIND_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_TYPE_GLOBAL_SHORTCUTS_REBIND_SKELETON))

typedef struct _CcGlobalShortcutsRebindSkeleton CcGlobalShortcutsRebindSkeleton;
typedef struct _CcGlobalShortcutsRebindSkeletonClass CcGlobalShortcutsRebindSkeletonClass;
typedef struct _CcGlobalShortcutsRebindSkeletonPrivate CcGlobalShortcutsRebindSkeletonPrivate;

struct _CcGlobalShortcutsRebindSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  CcGlobalShortcutsRebindSkeletonPrivate *priv;
};

struct _CcGlobalShortcutsRebindSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType cc_global_shortcuts_rebind_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcGlobalShortcutsRebindSkeleton, g_object_unref)
#endif

CcGlobalShortcutsRebind *cc_global_shortcuts_rebind_skeleton_new (void);


G_END_DECLS

#endif /* __CC_GLOBAL_SHORTCUTS_REBIND_GENERATED_H__ */
