
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "cc-number-row-enums.h"
#include "cc-number-row.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "cc-number-row.h" */

GType
cc_number_order_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CC_NUMBER_ORDER_FIRST), "CC_NUMBER_ORDER_FIRST", "first" },
        { C_ENUM (CC_NUMBER_ORDER_DEFAULT), "CC_NUMBER_ORDER_DEFAULT", "default" },
        { C_ENUM (CC_NUMBER_ORDER_LAST), "CC_NUMBER_ORDER_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("CcNumberOrder"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
cc_number_value_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CC_NUMBER_VALUE_CUSTOM), "CC_NUMBER_VALUE_CUSTOM", "custom" },
        { C_ENUM (CC_NUMBER_VALUE_STRING), "CC_NUMBER_VALUE_STRING", "string" },
        { C_ENUM (CC_NUMBER_VALUE_SECONDS), "CC_NUMBER_VALUE_SECONDS", "seconds" },
        { C_ENUM (CC_NUMBER_VALUE_MINUTES), "CC_NUMBER_VALUE_MINUTES", "minutes" },
        { C_ENUM (CC_NUMBER_VALUE_HOURS), "CC_NUMBER_VALUE_HOURS", "hours" },
        { C_ENUM (CC_NUMBER_VALUE_DAYS), "CC_NUMBER_VALUE_DAYS", "days" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("CcNumberValueType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
cc_number_sort_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (CC_NUMBER_SORT_ASCENDING), "CC_NUMBER_SORT_ASCENDING", "ascending" },
        { C_ENUM (CC_NUMBER_SORT_DESCENDING), "CC_NUMBER_SORT_DESCENDING", "descending" },
        { C_ENUM (CC_NUMBER_SORT_NONE), "CC_NUMBER_SORT_NONE", "none" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("CcNumberSortType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

