
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "bolt-enums.h" */

GType bolt_status_get_type (void);
#define BOLT_TYPE_STATUS (bolt_status_get_type())

GType bolt_auth_flags_get_type (void);
#define BOLT_TYPE_AUTH_FLAGS (bolt_auth_flags_get_type())

GType bolt_key_state_get_type (void);
#define BOLT_TYPE_KEY_STATE (bolt_key_state_get_type())

GType bolt_security_get_type (void);
#define BOLT_TYPE_SECURITY (bolt_security_get_type())

GType bolt_policy_get_type (void);
#define BOLT_TYPE_POLICY (bolt_policy_get_type())

GType bolt_auth_ctrl_get_type (void);
#define BOLT_TYPE_AUTH_CTRL (bolt_auth_ctrl_get_type())

GType bolt_device_type_get_type (void);
#define BOLT_TYPE_DEVICE_TYPE (bolt_device_type_get_type())

GType bolt_auth_mode_get_type (void);
#define BOLT_TYPE_AUTH_MODE (bolt_auth_mode_get_type())

/* enumerations from "bolt-error.h" */

GType bolt_error_get_type (void);
#define BOLT_TYPE_ERROR (bolt_error_get_type())

G_END_DECLS

/* Generated data ends here */

