
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "gdesktop-enums.h" */

GType g_desktop_proxy_mode_get_type (void);
#define G_DESKTOP_TYPE_PROXY_MODE (g_desktop_proxy_mode_get_type())

GType g_desktop_toolbar_style_get_type (void);
#define G_DESKTOP_TYPE_TOOLBAR_STYLE (g_desktop_toolbar_style_get_type())

GType g_desktop_toolbar_icon_size_get_type (void);
#define G_DESKTOP_TYPE_TOOLBAR_ICON_SIZE (g_desktop_toolbar_icon_size_get_type())

GType g_desktop_background_style_get_type (void);
#define G_DESKTOP_TYPE_BACKGROUND_STYLE (g_desktop_background_style_get_type())

GType g_desktop_background_shading_get_type (void);
#define G_DESKTOP_TYPE_BACKGROUND_SHADING (g_desktop_background_shading_get_type())

GType g_desktop_mouse_dwell_mode_get_type (void);
#define G_DESKTOP_TYPE_MOUSE_DWELL_MODE (g_desktop_mouse_dwell_mode_get_type())

GType g_desktop_mouse_dwell_direction_get_type (void);
#define G_DESKTOP_TYPE_MOUSE_DWELL_DIRECTION (g_desktop_mouse_dwell_direction_get_type())

GType g_desktop_clock_format_get_type (void);
#define G_DESKTOP_TYPE_CLOCK_FORMAT (g_desktop_clock_format_get_type())

GType g_desktop_screensaver_mode_get_type (void);
#define G_DESKTOP_TYPE_SCREENSAVER_MODE (g_desktop_screensaver_mode_get_type())

GType g_desktop_magnifier_mouse_tracking_mode_get_type (void);
#define G_DESKTOP_TYPE_MAGNIFIER_MOUSE_TRACKING_MODE (g_desktop_magnifier_mouse_tracking_mode_get_type())

GType g_desktop_magnifier_focus_tracking_mode_get_type (void);
#define G_DESKTOP_TYPE_MAGNIFIER_FOCUS_TRACKING_MODE (g_desktop_magnifier_focus_tracking_mode_get_type())

GType g_desktop_magnifier_caret_tracking_mode_get_type (void);
#define G_DESKTOP_TYPE_MAGNIFIER_CARET_TRACKING_MODE (g_desktop_magnifier_caret_tracking_mode_get_type())

GType g_desktop_magnifier_screen_position_get_type (void);
#define G_DESKTOP_TYPE_MAGNIFIER_SCREEN_POSITION (g_desktop_magnifier_screen_position_get_type())

GType g_desktop_titlebar_action_get_type (void);
#define G_DESKTOP_TYPE_TITLEBAR_ACTION (g_desktop_titlebar_action_get_type())

GType g_desktop_focus_mode_get_type (void);
#define G_DESKTOP_TYPE_FOCUS_MODE (g_desktop_focus_mode_get_type())

GType g_desktop_focus_new_windows_get_type (void);
#define G_DESKTOP_TYPE_FOCUS_NEW_WINDOWS (g_desktop_focus_new_windows_get_type())

GType g_desktop_visual_bell_type_get_type (void);
#define G_DESKTOP_TYPE_VISUAL_BELL_TYPE (g_desktop_visual_bell_type_get_type())

GType g_desktop_location_accuracy_level_get_type (void);
#define G_DESKTOP_TYPE_LOCATION_ACCURACY_LEVEL (g_desktop_location_accuracy_level_get_type())

GType g_desktop_touchpad_handedness_get_type (void);
#define G_DESKTOP_TYPE_TOUCHPAD_HANDEDNESS (g_desktop_touchpad_handedness_get_type())

GType g_desktop_touchpad_click_method_get_type (void);
#define G_DESKTOP_TYPE_TOUCHPAD_CLICK_METHOD (g_desktop_touchpad_click_method_get_type())

GType g_desktop_touchpad_tap_button_map_get_type (void);
#define G_DESKTOP_TYPE_TOUCHPAD_TAP_BUTTON_MAP (g_desktop_touchpad_tap_button_map_get_type())

GType g_desktop_device_send_events_get_type (void);
#define G_DESKTOP_TYPE_DEVICE_SEND_EVENTS (g_desktop_device_send_events_get_type())

GType g_desktop_tablet_mapping_get_type (void);
#define G_DESKTOP_TYPE_TABLET_MAPPING (g_desktop_tablet_mapping_get_type())

GType g_desktop_pad_button_action_get_type (void);
#define G_DESKTOP_TYPE_PAD_BUTTON_ACTION (g_desktop_pad_button_action_get_type())

GType g_desktop_stylus_button_action_get_type (void);
#define G_DESKTOP_TYPE_STYLUS_BUTTON_ACTION (g_desktop_stylus_button_action_get_type())

GType g_desktop_stylus_eraser_button_mode_get_type (void);
#define G_DESKTOP_TYPE_STYLUS_ERASER_BUTTON_MODE (g_desktop_stylus_eraser_button_mode_get_type())

GType g_desktop_pointer_accel_profile_get_type (void);
#define G_DESKTOP_TYPE_POINTER_ACCEL_PROFILE (g_desktop_pointer_accel_profile_get_type())

GType g_desktop_usb_protection_get_type (void);
#define G_DESKTOP_TYPE_USB_PROTECTION (g_desktop_usb_protection_get_type())

GType g_desktop_font_antialiasing_mode_get_type (void);
#define G_DESKTOP_TYPE_FONT_ANTIALIASING_MODE (g_desktop_font_antialiasing_mode_get_type())

GType g_desktop_font_hinting_get_type (void);
#define G_DESKTOP_TYPE_FONT_HINTING (g_desktop_font_hinting_get_type())

GType g_desktop_font_rgba_order_get_type (void);
#define G_DESKTOP_TYPE_FONT_RGBA_ORDER (g_desktop_font_rgba_order_get_type())

GType g_desktop_color_scheme_get_type (void);
#define G_DESKTOP_TYPE_COLOR_SCHEME (g_desktop_color_scheme_get_type())

GType g_desktop_pointing_stick_scroll_method_get_type (void);
#define G_DESKTOP_TYPE_POINTING_STICK_SCROLL_METHOD (g_desktop_pointing_stick_scroll_method_get_type())

GType g_desktop_font_rendering_get_type (void);
#define G_DESKTOP_TYPE_FONT_RENDERING (g_desktop_font_rendering_get_type())

GType g_desktop_accent_color_get_type (void);
#define G_DESKTOP_TYPE_ACCENT_COLOR (g_desktop_accent_color_get_type())

GType g_desktop_reduced_motion_get_type (void);
#define G_DESKTOP_TYPE_REDUCED_MOTION (g_desktop_reduced_motion_get_type())

GType g_desktop_weekday_get_type (void);
#define G_DESKTOP_TYPE_WEEKDAY (g_desktop_weekday_get_type())

G_END_DECLS

/* Generated data ends here */

