/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __CC_DBUS_COLOR_MANAGER_H__
#define __CC_DBUS_COLOR_MANAGER_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.ColorManager */

#define CC_DBUS_TYPE_COLOR_MANAGER (cc_dbus_color_manager_get_type ())
#define CC_DBUS_COLOR_MANAGER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER, CcDBusColorManager))
#define CC_DBUS_IS_COLOR_MANAGER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER))
#define CC_DBUS_COLOR_MANAGER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), CC_DBUS_TYPE_COLOR_MANAGER, CcDBusColorManagerIface))

struct _CcDBusColorManager;
typedef struct _CcDBusColorManager CcDBusColorManager;
typedef struct _CcDBusColorManagerIface CcDBusColorManagerIface;

struct _CcDBusColorManagerIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_calibrate_monitor) (
    CcDBusColorManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_connector);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManager, g_object_unref)
#endif

GType cc_dbus_color_manager_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *cc_dbus_color_manager_interface_info (void);
guint cc_dbus_color_manager_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void cc_dbus_color_manager_complete_calibrate_monitor (
    CcDBusColorManager *object,
    GDBusMethodInvocation *invocation,
    const gchar *session);



/* D-Bus method calls: */
void cc_dbus_color_manager_call_calibrate_monitor (
    CcDBusColorManager *proxy,
    const gchar *arg_connector,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean cc_dbus_color_manager_call_calibrate_monitor_finish (
    CcDBusColorManager *proxy,
    gchar ** out_session,
    GAsyncResult *res,
    GError **error);

gboolean cc_dbus_color_manager_call_calibrate_monitor_sync (
    CcDBusColorManager *proxy,
    const gchar *arg_connector,
    gchar ** out_session,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define CC_DBUS_TYPE_COLOR_MANAGER_PROXY (cc_dbus_color_manager_proxy_get_type ())
#define CC_DBUS_COLOR_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER_PROXY, CcDBusColorManagerProxy))
#define CC_DBUS_COLOR_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_DBUS_TYPE_COLOR_MANAGER_PROXY, CcDBusColorManagerProxyClass))
#define CC_DBUS_COLOR_MANAGER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_DBUS_TYPE_COLOR_MANAGER_PROXY, CcDBusColorManagerProxyClass))
#define CC_DBUS_IS_COLOR_MANAGER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER_PROXY))
#define CC_DBUS_IS_COLOR_MANAGER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_DBUS_TYPE_COLOR_MANAGER_PROXY))

typedef struct _CcDBusColorManagerProxy CcDBusColorManagerProxy;
typedef struct _CcDBusColorManagerProxyClass CcDBusColorManagerProxyClass;
typedef struct _CcDBusColorManagerProxyPrivate CcDBusColorManagerProxyPrivate;

struct _CcDBusColorManagerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  CcDBusColorManagerProxyPrivate *priv;
};

struct _CcDBusColorManagerProxyClass
{
  GDBusProxyClass parent_class;
};

GType cc_dbus_color_manager_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManagerProxy, g_object_unref)
#endif

void cc_dbus_color_manager_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcDBusColorManager *cc_dbus_color_manager_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
CcDBusColorManager *cc_dbus_color_manager_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void cc_dbus_color_manager_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcDBusColorManager *cc_dbus_color_manager_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
CcDBusColorManager *cc_dbus_color_manager_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define CC_DBUS_TYPE_COLOR_MANAGER_SKELETON (cc_dbus_color_manager_skeleton_get_type ())
#define CC_DBUS_COLOR_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER_SKELETON, CcDBusColorManagerSkeleton))
#define CC_DBUS_COLOR_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_DBUS_TYPE_COLOR_MANAGER_SKELETON, CcDBusColorManagerSkeletonClass))
#define CC_DBUS_COLOR_MANAGER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_DBUS_TYPE_COLOR_MANAGER_SKELETON, CcDBusColorManagerSkeletonClass))
#define CC_DBUS_IS_COLOR_MANAGER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER_SKELETON))
#define CC_DBUS_IS_COLOR_MANAGER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_DBUS_TYPE_COLOR_MANAGER_SKELETON))

typedef struct _CcDBusColorManagerSkeleton CcDBusColorManagerSkeleton;
typedef struct _CcDBusColorManagerSkeletonClass CcDBusColorManagerSkeletonClass;
typedef struct _CcDBusColorManagerSkeletonPrivate CcDBusColorManagerSkeletonPrivate;

struct _CcDBusColorManagerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  CcDBusColorManagerSkeletonPrivate *priv;
};

struct _CcDBusColorManagerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType cc_dbus_color_manager_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManagerSkeleton, g_object_unref)
#endif

CcDBusColorManager *cc_dbus_color_manager_skeleton_new (void);


/* ------------------------------------------------------------------------ */
/* Declarations for org.gnome.Mutter.ColorManager.Calibration */

#define CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION (cc_dbus_color_manager_calibration_get_type ())
#define CC_DBUS_COLOR_MANAGER_CALIBRATION(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION, CcDBusColorManagerCalibration))
#define CC_DBUS_IS_COLOR_MANAGER_CALIBRATION(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION))
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION, CcDBusColorManagerCalibrationIface))

struct _CcDBusColorManagerCalibration;
typedef struct _CcDBusColorManagerCalibration CcDBusColorManagerCalibration;
typedef struct _CcDBusColorManagerCalibrationIface CcDBusColorManagerCalibrationIface;

struct _CcDBusColorManagerCalibrationIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_set_crtc_gamma_lut) (
    CcDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue);

  gboolean (*handle_stop) (
    CcDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);

  guint  (*get_gamma_lut_size) (CcDBusColorManagerCalibration *object);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManagerCalibration, g_object_unref)
#endif

GType cc_dbus_color_manager_calibration_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *cc_dbus_color_manager_calibration_interface_info (void);
guint cc_dbus_color_manager_calibration_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void cc_dbus_color_manager_calibration_complete_set_crtc_gamma_lut (
    CcDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);

void cc_dbus_color_manager_calibration_complete_stop (
    CcDBusColorManagerCalibration *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void cc_dbus_color_manager_calibration_call_set_crtc_gamma_lut (
    CcDBusColorManagerCalibration *proxy,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean cc_dbus_color_manager_calibration_call_set_crtc_gamma_lut_finish (
    CcDBusColorManagerCalibration *proxy,
    GAsyncResult *res,
    GError **error);

gboolean cc_dbus_color_manager_calibration_call_set_crtc_gamma_lut_sync (
    CcDBusColorManagerCalibration *proxy,
    GVariant *arg_red,
    GVariant *arg_green,
    GVariant *arg_blue,
    GCancellable *cancellable,
    GError **error);

void cc_dbus_color_manager_calibration_call_stop (
    CcDBusColorManagerCalibration *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean cc_dbus_color_manager_calibration_call_stop_finish (
    CcDBusColorManagerCalibration *proxy,
    GAsyncResult *res,
    GError **error);

gboolean cc_dbus_color_manager_calibration_call_stop_sync (
    CcDBusColorManagerCalibration *proxy,
    GCancellable *cancellable,
    GError **error);



/* D-Bus property accessors: */
guint cc_dbus_color_manager_calibration_get_gamma_lut_size (CcDBusColorManagerCalibration *object);
void cc_dbus_color_manager_calibration_set_gamma_lut_size (CcDBusColorManagerCalibration *object, guint value);


/* ---- */

#define CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY (cc_dbus_color_manager_calibration_proxy_get_type ())
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, CcDBusColorManagerCalibrationProxy))
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, CcDBusColorManagerCalibrationProxyClass))
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY, CcDBusColorManagerCalibrationProxyClass))
#define CC_DBUS_IS_COLOR_MANAGER_CALIBRATION_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY))
#define CC_DBUS_IS_COLOR_MANAGER_CALIBRATION_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_PROXY))

typedef struct _CcDBusColorManagerCalibrationProxy CcDBusColorManagerCalibrationProxy;
typedef struct _CcDBusColorManagerCalibrationProxyClass CcDBusColorManagerCalibrationProxyClass;
typedef struct _CcDBusColorManagerCalibrationProxyPrivate CcDBusColorManagerCalibrationProxyPrivate;

struct _CcDBusColorManagerCalibrationProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  CcDBusColorManagerCalibrationProxyPrivate *priv;
};

struct _CcDBusColorManagerCalibrationProxyClass
{
  GDBusProxyClass parent_class;
};

GType cc_dbus_color_manager_calibration_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManagerCalibrationProxy, g_object_unref)
#endif

void cc_dbus_color_manager_calibration_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcDBusColorManagerCalibration *cc_dbus_color_manager_calibration_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
CcDBusColorManagerCalibration *cc_dbus_color_manager_calibration_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void cc_dbus_color_manager_calibration_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
CcDBusColorManagerCalibration *cc_dbus_color_manager_calibration_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
CcDBusColorManagerCalibration *cc_dbus_color_manager_calibration_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON (cc_dbus_color_manager_calibration_skeleton_get_type ())
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, CcDBusColorManagerCalibrationSkeleton))
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, CcDBusColorManagerCalibrationSkeletonClass))
#define CC_DBUS_COLOR_MANAGER_CALIBRATION_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON, CcDBusColorManagerCalibrationSkeletonClass))
#define CC_DBUS_IS_COLOR_MANAGER_CALIBRATION_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON))
#define CC_DBUS_IS_COLOR_MANAGER_CALIBRATION_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), CC_DBUS_TYPE_COLOR_MANAGER_CALIBRATION_SKELETON))

typedef struct _CcDBusColorManagerCalibrationSkeleton CcDBusColorManagerCalibrationSkeleton;
typedef struct _CcDBusColorManagerCalibrationSkeletonClass CcDBusColorManagerCalibrationSkeletonClass;
typedef struct _CcDBusColorManagerCalibrationSkeletonPrivate CcDBusColorManagerCalibrationSkeletonPrivate;

struct _CcDBusColorManagerCalibrationSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  CcDBusColorManagerCalibrationSkeletonPrivate *priv;
};

struct _CcDBusColorManagerCalibrationSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType cc_dbus_color_manager_calibration_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (CcDBusColorManagerCalibrationSkeleton, g_object_unref)
#endif

CcDBusColorManagerCalibration *cc_dbus_color_manager_calibration_skeleton_new (void);


G_END_DECLS

#endif /* __CC_DBUS_COLOR_MANAGER_H__ */
